<?php
// $Id$

/**
 * @file
 * Admin functions for the CONTENTdm Integration module.
 */

/**
 * Generate the admin settings form.
 */
function contentdm_admin_settings() {
  global $base_url;

  $form['contentdm_collections'] = array(
    '#type' => 'fieldset',
    '#title' => t('CONTENTdm collections'),
    '#collapsible' => TRUE, 
    '#collapsed' => TRUE 
   );

  $form['contentdm_collections']['overview_text'] = array(
    '#type' => 'markup', 
    '#value' => t("Note: No collections will appear in this section until you have completed the 
      'CONTENTdm API Settings' section and saved this form. Also, when ever you modify this list, 
      you should also clear the CONTENTdm cache using the linkat the bottom of the admin form."),
    '#prefix' => '<div>',
    '#suffix' => '</div>'
  );

  // We can't put a warning here about no collections being selected (like in _validate(), below) because if we do,
  // the drupal_set_message() will be repeated on validation. So, we choose to check and warn at validation.   
  $contentdm_all_collections = contentdm_get_all_collections();
  $form['contentdm_collections']['contentdm_collections_to_search'] = array(
    '#title' => 'Collections to search (including restricted collections)',
    '#type' => 'checkboxes',
    '#options' => $contentdm_all_collections,
    '#default_value' => variable_get('contentdm_collections_to_search', $contentdm_all_collections),
    '#description' => t('Note: This list includes all the published collections on your server, including 
      ones that might be restricted to some users. You should only select ones that do not have any access restrictions.')
  );

  $form['contentdm_api'] = array(
    '#type' => 'fieldset',
    '#title' => t('CONTENTdm API settings'),
    '#collapsible' => TRUE, 
    '#collapsed' => FALSE 
   );

  $form['contentdm_api']['contentdm_web_services_url'] = array(
    '#title' => t('Your CONTENTdm web services base URL'),
    '#type' => 'textfield',
    '#size' => 100,
    '#default_value' => variable_get('contentdm_web_services_url', 'http://contentdm.example.com:81/dmwebservices/index.php?q='),
    '#description' => t("The URL to your CONTENTdm web services base URL. Be sure to include the port number (by default, :81) 
      and index.php?q= at the end of your web services URL. If you are using a reverse proxy to provide access to a restricted 
      collection, you should also enable the CONTENTdm Integration reverse proxy URL filter."),
    '#maxlength' => 255,
    '#required' => TRUE
  );

  $form['contentdm_api']['contentdm_utils_base_url'] = array(
    '#title' => t("Your CONTENTdm 'utils' base URL"),
    '#type' => 'textfield',
    '#size' => 100,
    '#default_value' => variable_get('contentdm_utils_base_url', 'http://contentdm.example.com/utils/'),
    '#description' => t("The URL to your CONTENTdm 'utils' base URL. You do not need to include a port number unless it is different than :80. Be sure to include the trailing slash. If you are using a reverse proxy to provide access to a restricted collection, you should also enable the CONTENTdm Integration reverse proxy URL filter."),
    '#maxlength' => 255,
    '#required' => TRUE
  );

  $form['contentdm_api']['contentdm_web_services_timeout_length'] = array(
    '#title' => t('Search/browse timeout'),
    '#type' => 'textfield',
    '#size' => 4,
    '#default_value' => variable_get('contentdm_web_services_timeout_length', '30'),
    '#description' => t("Amount of time, in seconds, before the user is told their query has timed out."),
    '#maxlength' => 4,
    '#required' => TRUE
  );

  $form['contentdm_api']['contentdm_web_services_error_message'] = array(
    '#title' => t('API error message'),
    '#type' => 'textfield',
    '#size' => 100,
    '#default_value' => variable_get('contentdm_web_services_error_message', 
      'Your query could not be completed. The site admistrator has been notified.'),
    '#description' => t("Message displayed to the user if Drupal cannot connect to 
       the CONTENTdm web services API or the query times out."),
    '#maxlength' => 100,
    '#required' => TRUE
  );

  $form['contentdm_api']['contentdm_web_services_error_watchdog'] = array(
    '#type' => 'checkbox',
    '#title' => t('Log connection and timeout warnings.'),
    '#default_value' => variable_get('contentdm_web_services_error_watchdog', 0),
  );

  $form['contentdm_api']['contentdm_web_services_error_email'] = array(
    '#type' => 'checkbox',
    '#title' => t('Email connection and timeout warnings to site admin.'),
    '#default_value' => variable_get('contentdm_web_services_error_email', 0),
  );

  $form['contentdm_api']['contentdm_web_services_log_on_server'] = array(
    '#type' => 'checkbox',
    '#title' => t('Log web services calls on the CONTENTdm server.'),
    '#default_value' => variable_get('contentdm_web_services_log_on_server', 0),
    '#description' => t("Enabling this will write a log file on your CONTENTdm 
       server as configured in the web services config.php file."),
  );

  $form['contentdm_api']['contentdm_curl_verifypeer'] = array(
    '#type' => 'checkbox',
    '#title' => t('Turn off SSL peer certificate verification.'),
    '#default_value' => variable_get('contentdm_curl_verifypeer', 0),
    '#description' => t("Enabling this if your CONTENTdm server uses HTTPS and you 
      are seeing errors about invalid certificates."),
  );

  $form['contentdm_links'] = array(
    '#type' => 'fieldset',
    '#title' => t('Links to your CONTENTdm site'),
    '#collapsible' => TRUE, 
    '#collapsed' => FALSE 
   );

  $form['contentdm_links']['contentdm_advanced_options_blocks_text'] = array(
    '#type' => 'markup', 
    '#value' => t("These options are used by the \"Advanced options\" !blocks that link to your CONTENTdm site.", 
      array('!blocks' => l('blocks', 'admin/build/block'))), 
    '#prefix' => '<div>',
    '#suffix' => '</div>'
  );

  $form['contentdm_links']['contentdm_server_base_url'] = array(
    '#title' => t("Your CONTENTdm site's base URL"),
    '#type' => 'textfield',
    '#size' => 100,
    '#default_value' => variable_get('contentdm_server_base_url', 'http://contentdm.example.com/'),
    '#maxlength' => 255,
    '#description' => t("Your CONTENTdm server's base URL, which is usually its hostname with 'http://' prepended. Do not include the trailing slash."),
    '#required' => TRUE
  );

  $form['contentdm_links']['contentdm_server_search_url'] = array(
    '#title' => t("Your CONTENTdm advanced search form URL"),
    '#type' => 'textfield',
    '#size' => 100,
    '#default_value' => variable_get('contentdm_server_search_url', 'http://contentdm.example.com/cdm4/search.php'),
    '#maxlength' => 255,
    '#description' => t("The URL to your CONTENTdm server's advanced search form. Used in the \"Advanced search options\" block."),
    '#required' => TRUE
  );

  $form['search_results'] = array(
    '#type' => 'fieldset',
    '#title' => t('Search results'),
    '#collapsible' => TRUE, 
    '#collapsed' => FALSE 
   );

  $form['search_results']['nickname_text'] = array(
    '#type' => 'markup', 
    '#value' => "The names that you need to use in the next two fields are taken from the 'Field nicknames' column 
       in each collection's entry in the " . l('collection info', 'admin/settings/contentdm/collection_info') . ' page.',
    '#prefix' => '<div>',
    '#suffix' => '</div>'
  );
   
  $form['search_results']['contentdm_fields_to_show'] = array(
    '#title' => t('Nicknames of Dublin Core fields to retrieve in search results'),
    '#type' => 'textfield',
    '#size' => 50,
    '#default_value' => variable_get('contentdm_fields_to_show', 'title!creato!date!subjec!descri'),
    '#maxlength' => 100,
    '#description' => t("Maximum of 5, separated by exclamation points. Collections should map a local field to each of the ones indicated here."),
    '#required' => TRUE
  );

  $form['search_results']['contentdm_sort_by'] = array(
    '#title' => t('Sort results by'),
    '#type' => 'textfield',
    '#size' => 50,
    '#default_value' => variable_get('contentdm_sort_by', 'title!date!creato'),
    '#maxlength' => 100,
    '#description' => t("Separated by exclamation points. These field nicknames must be a subset of 
      those in 'Fields to retrieve in search results', above. For descending sort, add 'reverse' as 
      the last item in the list."),
    '#required' => TRUE
  );

  $form['search_results']['contentdm_results_page_size'] = array(
    '#title' => t('Number of records to show in each page of search results'),
    '#type' => 'textfield',
    '#size' => 3,
    '#default_value' => variable_get('contentdm_results_page_size', '10'),
    '#maxlength' => 3,
  );

  $form['search_results']['contentdm_results_pager_group_size'] = array(
    '#title' => t('Number of pager links to show to the left and to the right of the current page number'),
    '#type' => 'textfield',
    '#size' => 3,
    '#default_value' => variable_get('contentdm_results_pager_group_size', 5),
    '#maxlength' => 3,
  );

  $form['search_results']['contentdm_results_log_searches'] = array(
    '#type' => 'checkbox',
    '#title' => t('Log searches. Timeouts and connection errors are logged first (i.e., searches 
      that result in a timeout are not logged as searches.)'),
    '#default_value' => variable_get('contentdm_results_log_searches', 1),
  );

  $form['browsing'] = array(
    '#type' => 'fieldset',
    '#title' => t('Browsing'),
    '#collapsible' => TRUE, 
    '#collapsed' => FALSE 
   );
  
  $form['browsing']['contentdm_browse_page_title'] = array(
    '#title' => t('The title of your browse list'),
    '#type' => 'textfield',
    '#size' => 90,
    '#default_value' => variable_get('contentdm_browse_page_title', 'Browse our digital collections'),
    '#maxlength' => 255,
  );

  $form['browsing']['contentdm_clear_browse_cache'] = array(
    '#prefix' => '<p>',
    '#suffix' => '</p>',
    '#type' => 'markup',
    '#value' => t("CONTENTdm caches pages produced by browsing. This cache is not cleared when cron.php runs; you must !clear.", 
      array('!clear' => l('clear it manually', 'contentdm_clear_browse_cache'))),
  );

  $form['browsing']['contentdm_use_browse_cache'] = array(
    '#type' => 'checkbox',
    '#title' => t('Cache browse pages.'),
    '#default_value' => variable_get('contentdm_use_browse_cache', '0'),
    '#description' => t('Browse list caching interferes with hook_contentdm_dmquery_alter(),
      so you will need to turn off caching here if you use modules that implement this hook.'),
  );
  
  $form['full_item_display'] = array(
    '#type' => 'fieldset',
    '#title' => t('Full item display'),
    '#collapsible' => TRUE, 
    '#collapsed' => FALSE 
  );

  $form['full_item_display']['contentdm_page_display_fulltext'] = array(
    '#type' => 'checkbox',
    '#title' => t('If item has full text, display it along with image.'),
    '#default_value' => variable_get('contentdm_page_display_fulltext', '1'),
  );

  $form['full_item_display']['contentdm_item_display_parent_metadata'] = array(
    '#type' => 'checkbox',
    '#title' => t("If item has a parent (i.e., it is a child item), display the parent metadata in a collapsible 
      div at the top of the item. Note: heavy use sites may want to turn this off, since it makes a web services 
      call to the CONTENTdm server every time an item is viewed and also creates the parent item's node if it doesn't 
      exist already."), '#default_value' => variable_get('contentdm_item_display_parent_metadata', '1'),
  );

  $form['full_item_display']['contentdm_item_metadata_display_collection_link'] = array(
    '#type' => 'checkbox',
    '#title' => t("Display a link to browse the item's collection as the first field in the metadata."),
    '#default_value' => variable_get('contentdm_item_metadata_display_collection_link', 1),
  );

  $form['full_item_display']['contentdm_item_metadata_is_bad'] = array(
    '#type' => 'checkbox',
    '#title' => t('If item metadata or structure are malformed, display a sensible error to the user (uncheck to display nothing).'),
    '#default_value' => variable_get('contentdm_item_metadata_is_bad', 1),
  );

  $form['wrapper_node_ownership'] = array(
    '#type' => 'fieldset',
    '#title' => t('CONTENTdm wrapper node ownership'),
    '#collapsible' => TRUE, 
    '#collapsed' => FALSE 
  );

  $form['wrapper_node_ownership']['contentdm_enable_wrapper_owners'] = array(
    '#type' => 'checkbox',
    '#title' => t("Assign ownership of CONTENTdm wrapper nodes to the user who first views them."),
    '#default_value' => variable_get('contentdm_enable_wrapper_owners', '0'),
      '#description' => t('If not checked, CONTENTdm wrapper nodes will be owned by the anonymous user. 
        You should only turn this on if you are using other modules that rely on node ownership, such as
        CONTENTdm Organic Groups.'),
  );

  $form['add_collection_taxoterm'] = array(
    '#type' => 'fieldset',
    '#title' => t('Collection taxonomy term assignment'),
    '#collapsible' => TRUE, 
    '#collapsed' => FALSE 
  );

  $form['add_collection_taxoterm']['contentdm_add_collection_taxoterm'] = array(
    '#type' => 'checkbox',
    '#title' => t("Automatically add an item's CONTENTdm collection name as a taxonomy term."),
    '#default_value' => variable_get('contentdm_add_collection_taxoterm', '0'),
  );

  $vocabs = taxonomy_get_vocabularies('contentdm_wrapper');
  if (count($vocabs)) {
    foreach ($vocabs as $vocab) {
      $contentdm_wrapper_vocabs[$vocab->vid] = $vocab->name;
    }
    $form['add_collection_taxoterm']['contentdm_add_collection_vocab'] = array(
      '#title' => 'Use this taxonomy',
      '#type' => 'select',
      '#options' => $contentdm_wrapper_vocabs,
      '#default_value' => variable_get('contentdm_add_collection_vocab', array()),
      '#description' => t('Select which vocabulary to add collection names to. Only vocabularies that 
        are configured to apply to content type CONTENTdm Item Wrapper show up in this list. If collection 
        names do not exist in this taxonomy, they are added automatically.'),
    );
  }
  else {
    $form['add_collection_taxoterm']['no_vocab_info'] = array(
      '#type' => 'markup',
      '#value' => t('No vocabularies are currently configured to apply to CONTENTdm Item Wrapper nodes.'),
    );
  }

  $form['automatic_metadata_updates'] = array(
    '#type' => 'fieldset',
    '#title' => t('Automatic metadata updates'),
    '#collapsible' => TRUE, 
    '#collapsed' => FALSE 
  );

  $form['automatic_metadata_updates']['contentdm_automatic_metadata_updates'] = array(
    '#type' => 'checkbox',
    '#title' => t("Check an item's last modified date every time a user views the item, and refetch 
      the full metadata and structure if item has been modified on the CONTENTdm server. Note: heavy 
      use sites may want to turn this off, since it makes a web services call to the CONTENTdm server 
      every time an item is viewed."),
    '#default_value' => variable_get('contentdm_automatic_metadata_updates', 0),
  );

  $form['automatic_metadata_updates']['contentdm_automatic_metadata_updates_log'] = array(
    '#type' => 'checkbox',
    '#title' => t("Log updates."),
    '#default_value' => variable_get('contentdm_automatic_metadata_updates_log', 0),
  );

  $form['nodeless_mode'] = array(
    '#type' => 'fieldset',
    '#title' => t('Nodeless mode'),
    '#collapsible' => TRUE, 
    '#collapsed' => FALSE, 
  );

  $form['nodeless_mode']['contentdm_nodeless_mode_stub_nid'] = array(
    '#title' => t('Nodeless mode stub node ID'),
    '#type' => 'textfield',
    '#size' => 10,
    '#default_value' => variable_get('contentdm_nodeless_mode_stub_nid', ''),
    '#maxlength' => 10,
    '#description' => t("Enter the node ID of the stub node. Leave blank if you are not using nodeless mode."),
  );

  $form['nodeless_mode']['nodeless_mode_info'] = array(
    '#type' => 'markup',
    '#value' => t("Nodeless mode reuses the same node for all CONTENTdm items. In this mode, new nodes are not created 
      for each item, the CONTENTdm item's image (or video, etc.) and metadata is displayed in the node body and is 
      not stored in the local Drupal database. Note that if you use nodeless mode, you should not let users leave comments or 
      tag items, and you cannot use CONTENTdm items in views, since all comments and tags would appear on the single node used
      to display all CONTENTdm items. See the main CONTENTdm Integration module's README.txt file for more information."),
  );

  $form['cache'] = array(
    '#type' => 'fieldset',
    '#title' => t('Cached data'),
    '#collapsible' => FALSE, 
    '#collapsed' => FALSE 
  );

  $form['cache']['contentdm_cache_info'] = array(
    '#type' => 'markup',
    '#value' => t("Cached CONTENTdm-specific data such as the collection alias-to-name mappings and the 
      collection browse list regenerated on each cron.php run. You should !regenerate_cache manually after you have 
      added to or modified the list of collections to search. You will also want to !clear_browse_results cache after loading new items 
      into CONTENTdm.", 
      array('!regenerate_cache' => l('regenerate the cache', 'contentdm_reset_cache'), 
        '!clear_browse_results' => l('clear the browse results cache', 'contentdm_clear_browse_cache'))),
  );

  return system_settings_form($form); 
}

/**
 * Validation frunction for admin settings form.
 */
function contentdm_admin_settings_validate($form, &$form_state) {
  if (!preg_match("/(\/|index.php\?q=)$/", trim($form_state['values']['contentdm_web_services_url']))) {
    form_set_error('contentdm_web_services_url', t("Please include the trailing slash, or index.php?q=, at the end of %server", 
      array('%server' => $form_state['values']['contentdm_web_services_url'])));
  }
  if (preg_match("/\/$/", trim($form_state['values']['contentdm_server_base_url']))) {
    form_set_error('contentdm_server_base_url', t("Please remove the trailing slash from %server", 
      array('%server' => $form_state['values']['contentdm_server_base_url'])));
  }

  $ws_url = $form_state['values']['contentdm_web_services_url'] . 'help';
  $services = drupal_http_request($ws_url);
  if ($services->code == '0') { // No such host
    form_set_error('contentdm_web_services_url', t("Can't find the CONTENTdm server's hostname, 
	  please verify that it is correct in the web-services base URL and CGI-BIN base URL fields,
	  and in the \"Links to your CONTENTdm site\" section below.", 
      array('%file' => $form_state['values']['contentdm_web_services_url'])));
  }
  if ($services->code == '404') {
    form_set_error('contentdm_web_services_url', t("Can't find the path %file, please verify that it exists at that location", 
      array('%file' => $form_state['values']['contentdm_web_services_url'])));
  }

  $selected_collections = array_unique(array_values($form_state['values']['contentdm_collections_to_search']));
  if ((count($selected_collections) == 1) && ($selected_collections[0] == '0')) {
    drupal_set_message(t('Please note: You will not be able to search or browse CONTENTdm items until 
    you have selected at least one collection under "CONTENTdm collections" fieldgroup below.'), 'warning');
  }
}

/**
 * Generate and display to admin detailed information on CONTENTdm collections.
 */
function contentdm_display_all_collection_info() {
  // drupal_add_css(drupal_get_path('module', 'contentdm') .'/contentdm.css');
  $output = t("This page lists details on all collections in your CONTENTdm installation that are published and that are 
    not restricted by IP address or username.");
  $collections = contentdm_dmGetCollectionList();

  // dmQuery variables
  $field = array('title');
  $searchstring = array();
  $sortby = array();
  $maxrecs = 1024;
  $start = 1;

  for ($n = 0; $n < count($collections); $n++) {
    $output .= '<div class="collection_info">';
    $output .= '<h3>"' . $collections[$n]["name"] . '"</h3><p />';
    $output .= "Alias: {$collections[$n]["alias"]}<br> Name: {$collections[$n]["name"]}<br> " . 
      "File system path: {$collections[$n]["path"]}<br>";
    $conf = contentdm_dmGetCollectionFieldInfo($collections[$n]["alias"]);
    $output .= '<h4>Field details:</h4>';
    $output .= '<table border="1">';
    $output .= "<tr><th>Field nickname</th><th>Name</th><th>Type</th><th>Size</th><th>Search</th>" .
      "<th>Hide</th><th>Vocab</th><th>DC mapping</th></tr>";
    for ($i = 0; $i < count($conf); $i++) {
      $output .= '<tr><td>' .  $conf[$i]["nick"] . '</td><td>' . $conf[$i]["name"] . '</td><td>' . $conf[$i]["type"] . '</td><td>' .
        $conf[$i]["size"] . '</td><td>' . $conf[$i]["search"] . '</td><td>' . $conf[$i]["hide"] . '</td><td>' . $conf[$i]["vocab"] .
          '</td><td>' . $conf[$i]["dc"] . '</td></tr>';
    }
    $output .= '</table>';
    $output .= '</div>';
  }
  return $output;
}


