<?php
// $Id$

/**
 * @file
 * Defines mappings from CONTENTdm collection fields to CCK fields that have
 * been added to the 'contentdm_wrapper' content type. Keys in the $mappings
 * array are CONTENTmd collection aliases, and within each of the arrays keyed
 * with aliases are arrays that map source fields to target fields. 'source'
 * defines the CONTENTdm metadata field nickname, and 'target' defines the CCK
 * field name. 'callback' is an optional key that names a function in another
 * module that performs some processing on the CONTENTdm source value (such as
 * making it all uppercase) before the value populates the target CCK field.
 * This simple 'preprocessing' mechanism allows developers to write small,
 * custom modules to manipulate incoming CONTENTdm field values. 
 * 
 * Each collection can contain multiple mappings, but within each collection,
 * each CONTENTdm source field must map to only one target CCK field. Only CCK
 * fields of widget type 'Text field' can be targets.
 *
 * This in-code mapping is to allow easy testing of this module; a GUI admin
 * interface that will allow definition of these mappings is on the to do list.
 * To find the nicknames of the source fields, visit
 * admin/settings/contentdm/collection_info and look in the 'Field nickname'
 * column for the collection you are defining the mappings for.
 */
function contentdm_extra_fields_mappings() {
  $mappings = array(
    '/mydogcoll' => array(
      array('source' => 'title', 'target' => 'field_foo'),
      // In this mapping, the CONTENTdm source field value is passed to
      // mymodule_filter_bar before field_baz is populated.
      array(
        'source' => 'rights',
        'callback' => 'mymodule_filter_bar',
        'target' => 'field_baz'
      ),
    ),
    '/mycatcoll' => array(
      array(
        'source' => 'subjec',
        'target' => 'field_foo'
      ),
      array(
        'source' => 'title',
        'target' => 'field_baz'
      ),
    ),
  );
  return $mappings;
}
