<?php
// $Id$

/**
 * @file
 * Configuration file used to for determining which viewer module is used for a given item based on
 * properties of $node.
 *
 * In most cases, contentdm_get_viewer() is called with $node as a parameter, but this function
 * is also called by the compound object viewer module to get the appropriate item-level viewer
 * for the first child in a compound object. In order to avoid passing the entire $node back to
 * get the viewer, we provide the ability to pass only the file extension using NULL as the first
 * parameter.
 *
 * You should not need to modify this file. If you want to add your own custom viewer module, you should
 * implement hook_contentdm_viewer_alter($viewer_theme_function, $node), as documented in README.txt.
 *
*/

function contentdm_get_viewer($node = NULL, $file_extension = NULL) {
  if ($node) {
    // The CONTENTdm item metadata element 'find' contains the name of the file for each simple
    // (non-compound) item; for compound items, it contains a CONTENTdm-generated file that ends in .cpd.
    $item_metadata = unserialize($node->metadata);
    $file_extension = contentdm_get_file_extension($node->find);
    // For compound objects, it might be useful to know if we're dealing with a 'Document', 'Document-PDF',
    // 'Monograph', 'Picture cube', or 'Postcard', the types of compound object CONTENTdm supports. This 
    // will let us invoke a separate viewer for each type of compound object.
    if ($file_extension == 'cpd') {
      // Grab the value of the structure's <type> element.
      $structure = simplexml_load_string($node->structure);
      $type = (array) $structure->type;
      $type = trim($type[0]);
    }
  }

  switch (TRUE) {
    case ($file_extension == 'cpd'):
      $viewer_theme_function = 'contentdm_compound_item_viewer';
      break;
    case (preg_match('/(jpg|jpeg|jp2|gif|png)/i', $file_extension));
      $viewer_theme_function = 'contentdm_image_item_viewer';
      break;
    case (preg_match('/(pdf|pdfpage)/i', $file_extension));
      $viewer_theme_function = 'contentdm_pdf_item_viewer';
      break;
    case (preg_match('/(mpeg|wmv|mov|mp3|flv|mp4|mpeg4|wav)/i', $file_extension));
      $viewer_theme_function = 'contentdm_media_item_viewer';
      break;
    case (preg_match('/url/i', $file_extension));
      $viewer_theme_function = 'contentdm_url_item_viewer';
      break;
    case (preg_match('/txt/i', $file_extension));
      $viewer_theme_function = 'contentdm_text_item_viewer';
      break;
    default:
      $viewer_theme_function = 'contentdm_default_item_viewer';
  }

  return $viewer_theme_function;
}

