<?php

/**
 * @file
 * CONTENTdm thumbnail field handler.
 */

class contentdm_views_handler_thumbnail extends views_handler_field_node {
  function query() {
    $this->ensure_my_table();
    $this->add_additional_fields();
  }

  function option_definition() {
    $options = parent::option_definition();
    $options['thumbnail'] = array('default' => TRUE);
    $options['width'] = array('default' => 64);
    $options['height'] = array('default' => 64);
    return $options;
  }

  /**
   * Provide link to node option.
   */
  function options_form(&$form, &$form_state) {
    parent::options_form($form, $form_state);
    $form['thumbnail'] = array(
      '#title' => t('Use CONTENTdm default thumbnail'),
      '#description' => t('Use CONTENTdm\'s default item thumbnail. If this is set the height/width options below have no effect.'),
      '#type' => 'checkbox',
      '#default_value' => !empty($this->options['thumbnail']),
    );
    $form['width'] = array(
      '#title' => t('Width'),
      '#description' => t('Width of image in pixels.'),
      '#type' => 'textfield',
      '#size' => 5,
      '#maxlength' => 5,
      '#default_value' => $this->options['width'],
    );
    $form['height'] = array(
      '#title' => t('Height'),
      '#description' => t('Height of image in pixels.'),
      '#type' => 'textfield',
      '#size' => 5,
      '#maxlength' => 5,
      '#default_value' => $this->options['height'],
    );
  }

  function render($values) {
    $alias = $values->{$this->aliases['alias']};
    $pointer = $values->{$this->aliases['pointer']};

    // If no alias or pointer values, we don't anything to do.
    if ( (empty($alias) && '0' != $alias) || (empty($pointer) && '0' != $pointer) ) {
      return;
    }

    // Get the images full-sized dimensions from CONTENTdm.
    $image_info = contentdm_dmGetImageInfo($alias, $pointer);

    // Use the CONTENTdm tumbnail if that option is selected in field config,
    // or if image_info has a zero dimension (indicates a non-image item).
    if (!empty($this->options['thumbnail']) || $image_info['width'] == 0 || $image_info['height'] == 0) {
      $values->{$this->field_alias} = theme('contentdm_thumbnail', $alias, $pointer);
    }
    else {
      $image_dimensions = contentdm_calculate_thumbnail_dimensions($alias, $pointer, $this->options['width'], $this->options['height']);
      $values->{$this->field_alias} = theme('contentdm_image', $alias, $pointer, $image_dimensions['scale'], $this->options['width'], $this->options['height']);
    }

    if ($values) {
      return $this->render_link($values->{$this->field_alias}, $values);
    }

  }
}

/**
 * Given an images full-size width and height, calculates the width, length, and scale
 * values to pass to CONTENTdm so it can serve up the image.
 */
function contentdm_calculate_thumbnail_dimensions($alias, $pointer, $width, $height) {
  $image_dimensions = array();
  $image_info = contentdm_dmGetImageInfo($alias, $pointer);
  $image_dimensions['scale'] = max($width, $height) / max($image_info['width'], $image_info['height']) * 100;
  $image_dimensions['width'] = round($width * $image_dimensions['scale'] / 100);
  $image_dimensions['height'] = round($height * $image_dimensions['scale'] / 100);
  return $image_dimensions;
}

